
/*	Copyright (C) G. Jespersen
	See Licence.txt.*/

#if !defined HISTORY_H
#define HISTORY_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Stdafx.h"

#define HIST_STORE_NAME _T("KDHist.dat")	// .dat
#define HIST_BACKUP_EXT  _T("kdh")
#define DEF_TIME 2000, 1, 1, 1, 1, 1		// for CTime constructor - def dst

/*
  Video Viewing History:

  A CHistoryItem stores data on a viewed video - date time viewed,
  time spent viewing. Its CVideo member specifies the Tree Topic 
  it is listed under.

  CHistory stores any number of CHistory items, and has a single
  instance in CDoc.

  Khan Academy may list a single Video under multiple topics.
  If the video is viewed, history items are made for each Topic 
  listing, but with a different time last viewed. These multiple 
  histories have their m_inMultiTopics flag set.

  Since CHistory is entirely user generated, it is stored on disk
  in the user's app data folder. Provisions are made for backing 
  up, restoring and repairing the CHistory.

  Khan Academy revises its Topics frequently, and the user imports
  these changes via the Update tab. Since the CHistoryItem contains
  a CVideo which contains a Topic listing, the item may no longer
  jive with the updated data. This requires a validation function
  to rebuild the history based on a videos Title only.
  Similarly, restoring a CHistory may also have Topic mix-ups.

*/

//////////////////////////////////////////////////////////////
// CHistoryArray - soratable CArray

class CHistoryItem;
class CHistoryArray : public CArray<CHistoryItem, CHistoryItem&>   
{
	public:
	void Sort();
	void SortMp4Count();
//	int * Search(CHistoryItem & hist);

	private:
	static int SortHistory(const void *, const void *);
	static int SortHistoryMp4Count(const void *, const void *);
};

//////////////////////////////////////////////////////////////
// CHistoryItem
class CHistoryItem
{
public:
	CHistoryItem();
	~CHistoryItem();

	CHistoryItem(const CHistoryItem & item);

	void	operator=(const CHistoryItem & item);
	BOOL	operator==(const CHistoryItem & item);
	BOOL	operator!=(const CHistoryItem & item);
	void	Copy(const CHistoryItem & item);
	void	Zero();
//	CTime	GetDefaultTime() { return CTime(1970, 1, 1, 1, 1, 1); }; // = 32461 bug east of gmt?
	CTime	GetDefaultTime() { return CTime(DEF_TIME); }; // = 946717261
	void	Read(CArchive & ar);
	void	Write(CArchive & ar);

	CVideo	m_video;
	CTime	m_lastView;
	int		m_totalViewTime;		// seconds
	BOOL	m_inMultiTopics;		// same vid in multiple topics per KA
};


//////////////////////////////////////////////////////////////
// CHistory
class CHistory
{
public:
	CHistory();
	~CHistory();
	CHistory(const CHistory & hist);
	void	operator=(const CHistory & hist);
	BOOL	operator==(const CHistory & hist);
	BOOL	operator!=(const CHistory & hist);
	void	Copy(const CHistory & hist);


	void	AddHistoryItem(CHistoryItem & hist);
	void	AddHistoryItemCheater(const CVideo & video);
	void	DeleteHistory(const CVideo & video);
	BOOL	GetHistoryDisplayData(const CVideo & video, 
								  double & timesViewed, CTime & lastViewed);
	BOOL	GetHistoryItem(const CVideo & video, CHistoryItem & hist, int & itemIndex);
	BOOL	BeenViewed(const CVideo & video);
	BOOL	GetLastViewedVideo(CVideo & lastViewed);
	int		NumHistoryItems() { return m_histAr.GetSize(); };
	int		NumUniqueHistoryItems();
	BOOL	HistoryExists(CVideo & video);
	int		TopicVideosViewed(const CString & treeTopic);

	BOOL	BackupHistory(CString bkPath);
	BOOL	RestoreHistory(CString bkPath);
	BOOL	RepairHistory();
	BOOL	MergeHistory(CString bkPath);

	BOOL	Read();
	BOOL	Read(CString path);
	BOOL	Write();
	BOOL	Write(CString path);

private:
	BOOL	HistoryExists(CHistoryItem & hist, int & index);
	BOOL	HistoryExists(CVideo & video, int & index);
	BOOL	VideoTitleExists(const CString & vidTitle);
	BOOL	VideoMp4Exists(const CVideo & vid);
	int		Validate(CString & msg);
	int		Merge(CHistory & mergeHistory, CHistory & newHistory, CString & msg);

private:
	CHistoryArray m_histAr;
	int		m_schema;
};

#endif // HISTORY_H

/*
CTime bug notes:

  see: http://connect.microsoft.com/VisualStudio/feedback/details/679396/ctime-constructor-causes-crash-in-certain-time-zones-only
  check CTimeSpan also-
  check regional settings
  check setlocale
  check GetAsSystemTime()
  check GetCurrentTime
  see: http://gilesey.wordpress.com/2012/12/30/initailizing-mfccrt-for-consumption-of-regional-settings-internationalizationc/

/////////////////////////////
from: http://computer-programming-forum.com/82-mfc/ed215208ebf600a6.htm

There was a bug in at least one version of M/S's C-standard library,
that ignored timezone settings that were present in the main O/S - we
eneded up with PST here in South Africa! The fix was to set the timezone
ENVIRONMENT VARIABLE in the registry. (Look at [HKEY_LOCAL_MACHINE\
SYSTEM\ CurrentControlSet\ Control\ Session  Manager\ Environment]
"TZ"="<whatever>" )

This was NOT present in all versions of the library - we sent from one
machine to another and the time jumped by 10 hours! One machine was OK,
and the other wrong. Attached is MFC test code to see if you have this
problem:
------------------------ CODE---------------
// CTIM.CPP - Test of CTime Setting

#include <AFX.H>
#include <stdio.h>
#include <time.h>
#include <stdlib.h>

void main( void )
{
   CTime    CurTime;
   time_t   SysSec;
   char*    pEnvVar;
   CurTime = CTime::GetCurrentTime();
   SysSec = CurTime.GetTime();
   pEnvVar = getenv("TZ");
   if (pEnvVar)
   {
      printf("According to LIBC, this system has TZ set to %s\n", pEnvVar);
      _tzset();
   }
   else
   {
      printf("According to LIBC, this system has no TZ value set\n");
   }
   printf( "_daylight = %d\n", _daylight );
   printf( "_timezone = %ld (%d min)\n", _timezone, (_timezone / 60) );
   printf( "_tzname[0] = %s\n", _tzname[0] );

   DWORD tZresult;
   BOOL  gotTZ = FALSE;
   TIME_ZONE_INFORMATION   tZone;
   tZresult = GetTimeZoneInformation( &tZone );
   printf("WIN32 Time zone data gives zone = ");
   if (tZresult == TIME_ZONE_ID_UNKNOWN)
   {
      printf("UNKNOWN ");
      gotTZ = TRUE;
   }
   else if (tZresult == TIME_ZONE_ID_STANDARD)
   {
      printf("STANDARD ");
      gotTZ = TRUE;
   }
   else if (tZresult == TIME_ZONE_ID_DAYLIGHT)
   {
      printf("DAYLIGHT_SAVINGS ");
      gotTZ = TRUE;
   }
   else
   {
      printf("**ERROR**\n");
   }

   if (gotTZ)
   {
      wprintf(L"(%s)\n", tZone.StandardName);
      printf("Timezone Bias= %ld minutes\n", tZone.Bias );
   }
   printf("CTime::GetCurrentTime() returns:\n");
   printf("  From LOCAL conversion: Date: %04d-%02d-%02d,", 
         CurTime.GetYear(), CurTime.GetMonth(), CurTime.GetDay());
   printf("  Time: %02d:%02d:%02d\n",
         CurTime.GetHour(), CurTime.GetMinute(), CurTime.GetSecond());
   printf("  From GMT conversion: Date: %04d-%02d-%02d,",
         CurTime.GetGmtTm()->tm_year + 1900, CurTime.GetGmtTm()->tm_mon + 1,
         CurTime.GetGmtTm()->tm_mday);
   printf("  Time: %02d:%02d:%02d\n",
         CurTime.GetGmtTm()->tm_hour, CurTime.GetGmtTm()->tm_min,
         CurTime.GetGmtTm()->tm_sec);
   printf("System time = %ld sec\n", SysSec );
   printf("Press return ");
   getchar();
}
------------------------END CODE ----------- 

*/